#ifndef	__HELPER_H__
#define	__HELPER_H__

#include <OS.h>
#include <String.h>

// kreate_sem()		: creates a semaphore
//		- thread_count,		uint32			: initial semaphore count,	input
//		- name,				const char *	: semaphore's name,			input
//		- error,			status_t &		: creation error code,		input/output
//		- errorString,		BString &		: creation error text,		output
// returns
//		- sem_id

inline sem_id kreate_sem(uint32	thread_count, const char *name, status_t &error, BString &errorString)
{
	sem_id res = B_ERROR;

	if (B_OK == error && B_OK > (res = ::create_sem(thread_count, name)))
		switch (error = res)
		{
		case B_BAD_VALUE:

			errorString = "failure at creating a semaphore: invalid thread_count value (less than 0)."; 
			break;

		case B_NO_MEMORY:

			errorString = "failure at creating a semaphore: not enough memory to allocate the semaphore's name.";
			break;

		case B_NO_MORE_SEMS:

			errorString = "failure at creating a semaphore: all semaphores are currently taken.";
			break;

		default:

			errorString = "failure at creating a semaphore: alien error code.";
		}

	return res;
}

// kreate_port()	: creates a port
//		- queue_length,		int32			: port's queue length,	input
//		- name,				const char *	: port's name,			input
//		- error,			status_t &		: creation error code,	input/output
//		- errorString,		BString &		: creation error text,	output
// returns
//		- port_id

inline port_id kreate_port(int32 queue_length, const char *name, status_t &error, BString &errorString)
{
	port_id res = B_ERROR;
	
	if (B_OK == error && B_OK > (res = ::create_port(queue_length, name)))
		switch (error = res)
		{
		case B_BAD_VALUE:

			errorString = "failure at creating a port: queue_length is too big or less than zero.";
			break;

		case B_NO_MORE_PORTS:

			errorString = "failure at creating a port: all ports are currently taken.";
			break;

		default:

			errorString = "failure at creating a port: alien error code.";
		}

	return res;
}

// kreate_area()	: creates an area
//		- name,			const char *	: area's name,				input
//		- addr,			void **			: area's addres,			input/output
//		- addr_spec,	uint32			: address specification,	input
//		- size,			uint32			: area size,				input
//		- lock,			uint32			: area memory lock policy,	input
//		- protection,	uint32			: protection mask,			input
//		- error,		status_t &		: creation error code,		input/output
//		- errorString,	BString &		: creation error text,		input
// returns
//		- area_id

inline area_id kreate_area(const char *name,
						   void **addr,
						   uint32 addr_spec,
						   uint32 size,
						   uint32 lock,
						   uint32 protection,
						   status_t &error,
						   BString &errorString)
{
	area_id res = B_ERROR;

	if (B_OK == error && B_OK > (res = ::create_area(name, addr, addr_spec, size, lock, protection)))
		switch (error = res)
		{
		case B_BAD_VALUE:

			errorString = "failure at creating an area: bad argument value.";
			break;

		case B_NO_MEMORY:

			errorString = "failure at creating an area: not enough memory for the operation.";
			break;

		case B_ERROR:

			errorString = "failure at creating an area: generic system error.";
			break;

		default:

			errorString = "failure at creating an area: alien error code.";
		}

	return res;
}

// class Color	: an rgb_color wrapper providing construcition initialization

class Color : public rgb_color
{
public:
	Color(uint8 param_r, uint8 param_g, uint8 param_b, uint8 param_a = 255)
	{
		red		= param_r;
		green	= param_g;
		blue	= param_b;
		alpha	= param_a;
	};
};

#endif	// __HELPER_H__
